@@ -33,19 +33,28 @@
 #include "main/dispatch.h"
 #include "main/varray.h"
 
+static unsigned
+element_size(union gl_vertex_format_user format)
+{
+   return _mesa_bytes_per_vertex_attrib(format.Size, format.Type);
+}
+
+static void
+init_attrib(struct glthread_attrib *attrib, int index, int size, GLenum type)
+{
+   attrib->Format = MESA_PACK_VFORMAT(type, size, 0, 0, 0);
+   attrib->ElementSize = element_size(attrib->Format);
+   attrib->RelativeOffset = 0;
+   attrib->BufferIndex = index;
+   attrib->Stride = attrib->ElementSize;
+   attrib->Divisor = 0;
+   attrib->EnabledAttribCount = 0;
+   attrib->Pointer = NULL;
+}
 
 void
 _mesa_glthread_reset_vao(struct glthread_vao *vao)
 {
-   static unsigned default_elem_size[VERT_ATTRIB_MAX] = {
-      [VERT_ATTRIB_NORMAL] = 12,
-      [VERT_ATTRIB_COLOR1] = 12,
-      [VERT_ATTRIB_FOG] = 4,
-      [VERT_ATTRIB_COLOR_INDEX] = 4,
-      [VERT_ATTRIB_EDGEFLAG] = 1,
-      [VERT_ATTRIB_POINT_SIZE] = 4,
-   };
-
    vao->CurrentElementBufferName = 0;
    vao->UserEnabled = 0;
    vao->Enabled = 0;
@@ -55,17 +64,29 @@ _mesa_glthread_reset_vao(struct glthread_vao *vao)
    vao->NonZeroDivisorMask = 0;
 
    for (unsigned i = 0; i < ARRAY_SIZE(vao->Attrib); i++) {
-      unsigned elem_size = default_elem_size[i];
-      if (!elem_size)
-         elem_size = 16;
-
-      vao->Attrib[i].ElementSize = elem_size;
-      vao->Attrib[i].RelativeOffset = 0;
-      vao->Attrib[i].BufferIndex = i;
-      vao->Attrib[i].Stride = elem_size;
-      vao->Attrib[i].Divisor = 0;
-      vao->Attrib[i].EnabledAttribCount = 0;
-      vao->Attrib[i].Pointer = NULL;
+      switch (i) {
+      case VERT_ATTRIB_NORMAL:
+         init_attrib(&vao->Attrib[i], i, 3, GL_FLOAT);
+         break;
+      case VERT_ATTRIB_COLOR1:
+         init_attrib(&vao->Attrib[i], i, 3, GL_FLOAT);
+         break;
+      case VERT_ATTRIB_FOG:
+         init_attrib(&vao->Attrib[i], i, 1, GL_FLOAT);
+         break;
+      case VERT_ATTRIB_COLOR_INDEX:
+         init_attrib(&vao->Attrib[i], i, 1, GL_FLOAT);
+         break;
+      case VERT_ATTRIB_EDGEFLAG:
+         init_attrib(&vao->Attrib[i], i, 1, GL_UNSIGNED_BYTE);
+         break;
+      case VERT_ATTRIB_POINT_SIZE:
+         init_attrib(&vao->Attrib[i], i, 1, GL_FLOAT);
+         break;
+      default:
+         init_attrib(&vao->Attrib[i], i, 4, GL_FLOAT);
+         break;
+      }
    }
 }
 
@@ -333,26 +354,18 @@ void _mesa_glthread_AttribDivisor(struct gl_context *ctx, const GLuint *vaobj,
       vao->NonZeroDivisorMask &= ~(1u << attrib);
 }
 
-static unsigned
-element_size(GLint size, GLenum type)
-{
-   if (size == GL_BGRA)
-      size = 4;
-
-   return _mesa_bytes_per_vertex_attrib(size, type);
-}
-
 static void
 attrib_pointer(struct glthread_state *glthread, struct glthread_vao *vao,
                GLuint buffer, gl_vert_attrib attrib,
-               GLint size, GLenum type, GLsizei stride,
+               union gl_vertex_format_user format, GLsizei stride,
                const void *pointer)
 {
    if (attrib >= VERT_ATTRIB_MAX)
       return;
 
-   unsigned elem_size = element_size(size, type);
+   unsigned elem_size = element_size(format);
 
+   vao->Attrib[attrib].Format = format;
    vao->Attrib[attrib].ElementSize = elem_size;
    vao->Attrib[attrib].Stride = stride ? stride : elem_size;
    vao->Attrib[attrib].Pointer = pointer;
@@ -373,21 +386,21 @@ attrib_pointer(struct glthread_state *glthread, struct glthread_vao *vao,
 
 void
 _mesa_glthread_AttribPointer(struct gl_context *ctx, gl_vert_attrib attrib,
-                             GLint size, GLenum type, GLsizei stride,
-                             const void *pointer)
+                             union gl_vertex_format_user format,
+                             GLsizei stride, const void *pointer)
 {
    struct glthread_state *glthread = &ctx->GLThread;
 
    attrib_pointer(glthread, glthread->CurrentVAO,
                   glthread->CurrentArrayBufferName,
-                  attrib, size, type, stride, pointer);
+                  attrib, format, stride, pointer);
 }
 
 void
 _mesa_glthread_DSAAttribPointer(struct gl_context *ctx, GLuint vaobj,
                                 GLuint buffer, gl_vert_attrib attrib,
-                                GLint size, GLenum type, GLsizei stride,
-                                GLintptr offset)
+                                union gl_vertex_format_user format,
+                                GLsizei stride, GLintptr offset)
 {
    struct glthread_state *glthread = &ctx->GLThread;
    struct glthread_vao *vao;
@@ -396,45 +409,48 @@ _mesa_glthread_DSAAttribPointer(struct gl_context *ctx, GLuint vaobj,
    if (!vao)
       return;
 
-   attrib_pointer(glthread, vao, buffer, attrib, size, type, stride,
+   attrib_pointer(glthread, vao, buffer, attrib, format, stride,
                   (const void*)offset);
 }
 
 static void
 attrib_format(struct glthread_state *glthread, struct glthread_vao *vao,
-              GLuint attribindex, GLint size, GLenum type,
+              GLuint attribindex, union gl_vertex_format_user format,
               GLuint relativeoffset)
 {
    if (attribindex >= VERT_ATTRIB_GENERIC_MAX)
       return;
 
-   unsigned elem_size = element_size(size, type);
+   unsigned elem_size = element_size(format);
 
    unsigned i = VERT_ATTRIB_GENERIC(attribindex);
+   vao->Attrib[i].Format = format;
    vao->Attrib[i].ElementSize = elem_size;
    vao->Attrib[i].RelativeOffset = relativeoffset;
 }
 
 void
 _mesa_glthread_AttribFormat(struct gl_context *ctx, GLuint attribindex,
-                            GLint size, GLenum type, GLuint relativeoffset)
+                            union gl_vertex_format_user format,
+                            GLuint relativeoffset)
 {
    struct glthread_state *glthread = &ctx->GLThread;
 
-   attrib_format(glthread, glthread->CurrentVAO, attribindex, size, type,
+   attrib_format(glthread, glthread->CurrentVAO, attribindex, format,
                  relativeoffset);
 }
 
 void
 _mesa_glthread_DSAAttribFormat(struct gl_context *ctx, GLuint vaobj,
-                               GLuint attribindex, GLint size, GLenum type,
+                               GLuint attribindex,
+                               union gl_vertex_format_user format,
                                GLuint relativeoffset)
 {
    struct glthread_state *glthread = &ctx->GLThread;
    struct glthread_vao *vao = lookup_vao(ctx, vaobj);
 
    if (vao)
-      attrib_format(glthread, vao, attribindex, size, type, relativeoffset);
+      attrib_format(glthread, vao, attribindex, format, relativeoffset);
 }
 
 static void
@@ -687,8 +703,10 @@ _mesa_glthread_InterleavedArrays(struct gl_context *ctx, GLenum format,
    /* Texcoords */
    if (layout.tflag) {
       _mesa_glthread_ClientState(ctx, NULL, tex, true);
-      _mesa_glthread_AttribPointer(ctx, tex, layout.tcomps, GL_FLOAT, stride,
-                                   (GLubyte *) pointer + layout.toffset);
+      _mesa_glthread_AttribPointer(ctx, tex,
+                                   MESA_PACK_VFORMAT(GL_FLOAT, layout.tcomps,
+                                                     0, 0, 0),
+                                   stride, (GLubyte *)pointer + layout.toffset);
    } else {
       _mesa_glthread_ClientState(ctx, NULL, tex, false);
    }
@@ -696,9 +714,10 @@ _mesa_glthread_InterleavedArrays(struct gl_context *ctx, GLenum format,
    /* Color */
    if (layout.cflag) {
       _mesa_glthread_ClientState(ctx, NULL, VERT_ATTRIB_COLOR0, true);
-      _mesa_glthread_AttribPointer(ctx, VERT_ATTRIB_COLOR0, layout.ccomps,
-                                   layout.ctype, stride,
-                                   (GLubyte *) pointer + layout.coffset);
+      _mesa_glthread_AttribPointer(ctx, VERT_ATTRIB_COLOR0,
+                                   MESA_PACK_VFORMAT(layout.ctype, layout.ccomps,
+                                                     1, 0, 0),
+                                   stride, (GLubyte *)pointer + layout.coffset);
    } else {
       _mesa_glthread_ClientState(ctx, NULL, VERT_ATTRIB_COLOR0, false);
    }
@@ -706,7 +725,8 @@ _mesa_glthread_InterleavedArrays(struct gl_context *ctx, GLenum format,
    /* Normals */
    if (layout.nflag) {
       _mesa_glthread_ClientState(ctx, NULL, VERT_ATTRIB_NORMAL, true);
-      _mesa_glthread_AttribPointer(ctx, VERT_ATTRIB_NORMAL, 3, GL_FLOAT,
+      _mesa_glthread_AttribPointer(ctx, VERT_ATTRIB_NORMAL,
+                                   MESA_PACK_VFORMAT(GL_FLOAT, 3, 1, 0, 0),
                                    stride, (GLubyte *) pointer + layout.noffset);
    } else {
       _mesa_glthread_ClientState(ctx, NULL, VERT_ATTRIB_NORMAL, false);
@@ -714,6 +734,42 @@ _mesa_glthread_InterleavedArrays(struct gl_context *ctx, GLenum format,
 
    /* Vertices */
    _mesa_glthread_ClientState(ctx, NULL, VERT_ATTRIB_POS, true);
-   _mesa_glthread_AttribPointer(ctx, VERT_ATTRIB_POS, layout.vcomps, GL_FLOAT,
+   _mesa_glthread_AttribPointer(ctx, VERT_ATTRIB_POS,
+                                MESA_PACK_VFORMAT(GL_FLOAT, layout.vcomps,
+                                                  0, 0, 0),
                                 stride, (GLubyte *) pointer + layout.voffset);
 }
+
+static void
+unbind_uploaded_vbos(void *_vao, void *_ctx)
+{
+   struct gl_context *ctx = _ctx;
+   struct gl_vertex_array_object *vao = _vao;
+
+   assert(ctx->API != API_OPENGL_CORE);
+
+   for (unsigned i = 0; i < ARRAY_SIZE(vao->BufferBinding); i++) {
+      if (vao->BufferBinding[i].BufferObj &&
+          vao->BufferBinding[i].BufferObj->GLThreadInternal) {
+         /* We don't need to restore the user pointer because it's never
+          * overwritten. When we bind a VBO internally, the user pointer
+          * in gl_array_attribute::Ptr becomes ignored and unchanged.
+          */
+         _mesa_bind_vertex_buffer(ctx, vao, i, NULL, 0,
+                                  vao->BufferBinding[i].Stride, false, false);
+      }
+   }
+}
+
+/* Unbind VBOs in all VAOs that glthread bound for non-VBO vertex uploads
+ * to restore original states.
+ */
+void
+_mesa_glthread_unbind_uploaded_vbos(struct gl_context *ctx)
+{
+   assert(ctx->API != API_OPENGL_CORE);
+
+   /* Iterate over all VAOs. */
+   _mesa_HashWalk(ctx->Array.Objects, unbind_uploaded_vbos, ctx);
+   unbind_uploaded_vbos(ctx->Array.DefaultVAO, ctx);
+}
